// This is a simple interface allowing sontrol of the Arduino
// by a very simple serial protocol.
// Currently it supports control of all pins (in and out) as
// well as timing.
//
// The protocol is simple ASCII with arguments in prefix notation.
// Commands are lowercase letters (a-z), values can be either expressed
// in decimal form or as one-character values (@A-Z). Two-argument
// commands can be expressed as 123,123 X,123 or 123X only
// (e.g. 123,X is currently unsupported!)
//
//
int inc = 0; // incoming character
int value = 0, value2 = 0; // currently we store at most two values (we don't need more yet)
boolean quiet = false; // in quiet mode no responses are sent for commands that don't expect a result (to save time, e.g.)
unsigned long last_discovery = 0; // time stamp of last discovery (or I/O)
unsigned long discovery_delay = 60000L; // delay between discovery messages
char discovery_prefix[] = "ARDUINO-SCI-A-"; // can be customized for each device
int i = 0;

void setup()
{
  Serial.begin(9600);
}

void loop()
{
  if (Serial.available() > 0) {
    inc = Serial.read();
    if (inc >= 0x30 && inc <= 0x39) { // 0-9 interpreted as decimal value
      value *= 10;
      value += inc - 0x30;
    } else if (inc >= 0x40 && inc <= 0x5a) { // @A-Z interpreted as one single value with preceding shift
      value2 = value; value = inc - 0x40;
    } else if (inc == ',') { // shift values
      value2 = value; value = 0;
    } else {
      // process any commands
      last_discovery = millis(); // this counts as activity so reset the discovery timeout
      switch (inc) {
        case 's': digitalWrite(value, HIGH); if (!quiet) Serial.print("s\n"); break;
        case 'z': digitalWrite(value, LOW); if (!quiet) Serial.print("z\n"); break;
        case 'w': i = 0; while (i++ < 8) pinMode(i + 1, OUTPUT); // set all pins to output mode
                  i = 0; while (i++ < 8) { digitalWrite(i + 2, ((value & 1) == 0) ? LOW : HIGH); value >>= 1; }
                  if (!quiet) Serial.print("w\n"); break;
        case 'l': i = 0; while (i++ < 8) pinMode(i + 1, INPUT); // set all pins to input mode
                  i = 0; value = 0; while (i++ < 8) { value <<= 1; value |= ((digitalRead(i + 1) == HIGH) ? 1 : 0); }
                  Serial.print(value); Serial.print("l\n"); break;
        case 'o': pinMode(value, OUTPUT); if (!quiet) Serial.print("o\n"); break;
        case 'r': pinMode(value, INPUT); Serial.print(digitalRead(value)); Serial.print("r\n"); break;
        case 'a': pinMode(value, OUTPUT); analogWrite(value, value2); if (!quiet) Serial.print("a\n"); break;
        case 'i': pinMode(14 + value, INPUT); Serial.print(analogRead(value)); Serial.print("i\n"); break;
        case 'v': Serial.print(discovery_prefix); Serial.print("1.0.0v\n"); break;
        case 'd': delay(value); if (!quiet) Serial.print("d\n"); break;
        case 't': Serial.print(millis()); Serial.print("t\n"); break;
        case 'y': discovery_delay = value; discovery_delay *= 1000L; if (discovery_delay == 0) discovery_delay = 3000L; if (!quiet) Serial.print("y\n"); break; 
        case 'q': quiet = (value != 0); if (!quiet) Serial.print("q\n"); break; 
        default:
          if (inc >= 'a' && inc <= 'z') Serial.print("?\n");
      }
      // reset the value after any non-value character
      value = 0;
    }
  } else if (millis() > last_discovery + discovery_delay) { // send a discovery message
    Serial.print(discovery_prefix); Serial.print(millis()); Serial.print("y\n");
    last_discovery = millis();
  }
}
