// NewHeaven COG test code

//
//                   +---\/---+
//  INT0 (D 0) PB0  1|        |40  PA0 (AI 0 / D31)
//  INT1 (D 1) PB1  2|        |39  PA1 (AI 1 / D30)
//  INT2 (D 2) PB2  3|        |38  PA2 (AI 2 / D29)
//   PWM (D 3) PB3  4|        |37  PA3 (AI 3 / D28)
//   PWM (D 4) PB4  5|        |36  PA4 (AI 4 / D27)
//  MOSI (D 5) PB5  6|        |35  PA5 (AI 5 / D26)
//  MISO (D 6) PB6  7|        |34  PA6 (AI 6 / D25)
//   SCK (D 7) PB7  8|        |33  PA7 (AI 7 / D24)
//             RST  9|        |32  AREF
//             VCC 10|  644P  |31  GND 
//             GND 11|        |30  AVCC
//           XTAL2 12|        |29  PC7 (D 23)
//           XTAL1 13|        |28  PC6 (D 22)
//  RX0 (D 8)  PD0 14|        |27  PC5 (D 21) TDI
//  TX0 (D 9)  PD1 15|        |26  PC4 (D 20) TDO
//  RX1 (D 10) PD2 16|        |25  PC3 (D 19) TMS
//  TX1 (D 11) PD3 17|        |24  PC2 (D 18) TCKn
//  PWM (D 12) PD4 18|        |23  PC1 (D 17) SDA
//  PWM (D 13) PD5 19|        |22  PC0 (D 16) SCL
//  PWM (D 14) PD6 20|        |21  PD7 (D 15) PWM
//                   +--------+


#define LCD_DATA_PORT PORTA
#define LCD_DATA_DDR  DDRA
#define LCD_DATA_IN   PINA

#define LCD_CTRL_PORT PORTC
#define LCD_CTRL_DDR  DDRC
#define LCD_CTRL_MASK ((unsigned char)0xf8)

#define E1   sbi(PORTC, PORTC7)
#define E0   cbi(PORTC, PORTC7)
#define RW1  sbi(PORTC, PORTC6)
#define RW0  cbi(PORTC, PORTC6)
#define A1   sbi(PORTC, PORTC5)
#define A0   cbi(PORTC, PORTC5)
#define RST1 sbi(PORTC, PORTC4)
#define RST0 cbi(PORTC, PORTC4)
#define CS1  sbi(PORTC, PORTC3)
#define CS0  cbi(PORTC, PORTC3)

#define FF ((unsigned char) 0xff)

#define nop __asm__("nop\n\t")
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif 

#define LCD_WRITE(X,Y) { \
  E0; Y; RW0; \
  LCD_DATA_DDR = FF; \
  LCD_DATA_PORT = x; \
  CS0; E1; nop; CS1; E0; \
}

static void lcd_write_data(unsigned char x) LCD_WRITE(x, A1)
static void lcd_write_cmd(unsigned char x) LCD_WRITE(x, A0)

// this is not working but we should not needed it ...
static unsigned char lcd_read() { E0; LCD_DATA_DDR = 0; LCD_DATA_PORT = FF; A0; RW1; CS0; E1; nop; unsigned char tmp = LCD_DATA_IN; E0; CS1; return tmp; }

static void lcd_goto(unsigned char col, unsigned char page) {
    lcd_write_cmd(0xb0 | page); // set page
    lcd_write_cmd(col & 15); lcd_write_cmd(0x10 | (col >> 4)); // set column
}

static void lcd_fill(unsigned char x) {
  for (unsigned char page = 0; page < 8; page ++) {
    lcd_goto(0, page);
    for (unsigned char col = 0; col < 128; col++) lcd_write_data(x);
  }
}

static void lcd_clear() { lcd_fill(0); }

void setup() {
  LCD_CTRL_DDR |= LCD_CTRL_MASK;
  RST0;
  LCD_DATA_DDR = FF;
  LCD_DATA_PORT = 0;
  RW0; A0; E0;
  Serial.begin(9600);
  delay(20);
  RST1;
  delay(20);
  // mostly from NewHeaven refs
  lcd_write_cmd(0xa2); // 1/9 bias
  lcd_write_cmd(0xc8); // output reverse
  lcd_write_cmd(0xa4); // all points normal
  lcd_write_cmd(0x40); // start line set
  lcd_write_cmd(0x25); // internal reistor ratio (20-27)
  lcd_write_cmd(0x81); // electronic volume mode set
  lcd_write_cmd(0x18); // electronic volume (0-3f)
  lcd_write_cmd(0x2f); // power ctrl set (all on)
  lcd_clear();
  lcd_write_cmd(0xaf); // display on
  Serial.println("display initialized");
  
  lcd_goto(0, 4);
  for (int i = 0; i < 128; i++) lcd_write_data(0xc3);
}

void loop() {
  delay(4000);
  lcd_write_cmd(0xa7);
  delay(4000);
  lcd_write_cmd(0xa6);
  delay(1000);
}
